 function score = rule_golden(I, object, bestLines)

    if (size(I,3) == 3),
        I = rgb2gray(I);
    end
    [h w] = size(I);
    
    %% initialize power points    
    power_points(1) = struct('X', 2/5*w, 'Y', 3/5*h);
    power_points(2) = struct('X', 3/5*w, 'Y', 3/5*h);
    power_points(3) = struct('X', 2/5*w, 'Y', 2/5*h);
    power_points(4) = struct('X', 3/5*w, 'Y', 2/5*h);

    %% initialize power lines
    power_lines(1) = struct('X1', 0, 'Y1', 3/5*h, 'X2', w, 'Y2', 3/5*h,'slope',0,'yint',3/5*h);
    power_lines(2) = struct('X1', 0, 'Y1', 2/5*h, 'X2', w, 'Y2', 2/5*h,'slope',0,'yint',2/5*h);
    power_lines(3) = struct('X1', 2/5*w, 'Y1', 0, 'X2', 2/5*w, 'Y2', h,'slope',Inf,'yint',Inf);
    power_lines(4) = struct('X1', 3/5*w, 'Y1', 0, 'X2', 3/5*w, 'Y2', h,'slope',Inf,'yint',Inf);

    %% Initialize other variables
    sig1            = .17;
    weightedMass    = 0;
    mass            = 0;

    %% Compute object score
    % for relative distancing...
    maxD = max(h,w)/3;
    DS = zeros(length(object),1);
    MS = zeros(length(object),1);
    % sal_centerX     = 0;
    % sal_centerY     = 0;
    % sal_weight      = 0;
    % numPixels       = h*w;

    for i = 1:length(object)
       %compute nearest power point to center of region
       DS(i) = maxD;          % the maximum distance any object can be from 

       % for all 4 power points
       for j=1:4
           dist = sqrt( (object(i).centerX - power_points(j).X)^2 + (object(i).centerY - power_points(j).Y)^2 );
           if (dist < DS(i))
               DS(i) = dist;
           end
       end

    %    sal_centerX  = sal_centerX + object(i).centerX*(object(i).size/numPixels);
    %    sal_centerY  = sal_centerY + object(i).centerY*(object(i).size/numPixels);
    %    sal_weight   = sal_weight + object(i).size/numPixels;

       DS(i) = DS(i)/maxD;
       MS(i) = object(i).size*object(i).salience;

       weightedMass = weightedMass + MS(i)*exp(-((DS(i))^2)/(2*sig1));
       mass         = mass + MS(i);
    end

    % sal_center = [sal_centerX / sal_weight, sal_centerY / sal_weight];

    Epoint = weightedMass/mass;

    %% Compute line score
    divisor     = 0;
    numerator   = 0;
    sig2 = .17;

    for i = 1:length(bestLines)
        %remove if diagonal line
        if (bestLines(i).type ~= 3)
            %compute nearest power line
            D = maxD;
            for j=1:4
                dist = lineDist(h,w,bestLines(i),power_lines(j));
                if (dist < D)
                   D = dist;
                end
            end        

            D = D/maxD;

            divisor = divisor + bestLines(i).salience;
            numerator = numerator + bestLines(i).salience * exp(-D^2/(2*sig2));
        end
    end

    Eline = numerator / divisor;

    %% putting the two scores together
    point_gamma     = 1/2;
    line_gamma      = 1/2;

    score = point_gamma * Epoint + line_gamma * Eline;

end