function [saliency_map, object_map] = spectralResidual(I,n,thresh_scale)
% This function is a based on the spectral residual approach described by
% Hou and Zhang.
%       I        = the input scaled grayscale image as a double
%       n        = the dimension of our filters
%       thresh_scale -> threshold scale, default is 3

%  Where A is the fft of I_rgb,
%       Pf = phase of fft
%       Lf = ln of the amplitude
%       Rf = Lf - (h_n convolved with Lf)

%  The resulting Saliency map becomes
%       S = gaussian filter conv with fft inverse of exp(Rf+iPf) squared

%% Set Default threshold scale
if nargin < 2, n = 3; end
if nargin < 3, thresh_scale = 3; end

%% Calculate the FFT of the image
A = fft2(I);

%% Calculate the Residual
% phase and log amplitude
Pf = angle(A);
Lf = log(abs(A));

% Spectral Residual
Rf = Lf - imfilter(Lf,fspecial('average', n),'replicate');

%% Calculate the Saliency map
% Saliency map
saliency_map  = abs(ifft2(exp(Rf + i.*Pf))).^2;
saliency_map  = imfilter(saliency_map, fspecial('disk', n));
saliency_map  = mat2gray(saliency_map);

%% Compute Object Map
S_avg  = mean(mean(saliency_map));
thresh = thresh_scale*S_avg;
object_map = zeros(size(saliency_map));
object_map(saliency_map > thresh) = 1;